@@ -1,15 +1,8 @@
-"""
-    weasyprint.tests.test_draw.test_image
-    -------------------------------------
-
-    Test how images are drawn.
-
-"""
+"""Test how images are drawn."""
 
 import pytest
 
 from ..testing_utils import assert_no_logs, capture_logs
-from . import assert_pixels, assert_same_rendering
 
 centered_image = '''
     ________
@@ -134,6 +127,17 @@ cover_image = '''
     ________
 '''
 
+border_image = '''
+    ________
+    _GGGGGG_
+    _GrBBBG_
+    _GBBBBG_
+    _GBBBBG_
+    _GBBBBG_
+    _GGGGGG_
+    ________
+'''
+
 
 @assert_no_logs
 @pytest.mark.parametrize('filename, image', (
@@ -143,11 +147,11 @@ cover_image = '''
     ('pattern.gif', centered_image),
     ('blue.jpg', blue_image)
 ))
-def test_images(filename, image):
-    assert_pixels(f'inline_image_{filename}', 8, 8, image, '''
+def test_images(assert_pixels, filename, image):
+    assert_pixels(image, '''
       <style>
         @page { size: 8px }
-        body { margin: 2px 0 0 2px; background: #fff; font-size: 0 }
+        body { margin: 2px 0 0 2px; font-size: 0 }
       </style>
       <div><img src="%s"></div>''' % filename)
 
@@ -159,11 +163,11 @@ def test_images(filename, image):
     'pattern.palette.png',
     'pattern.gif',
 ))
-def test_resized_images(filename):
-    assert_pixels(f'resized_image_{filename}', 12, 12, resized_image, '''
+def test_resized_images(assert_pixels, filename):
+    assert_pixels(resized_image, '''
       <style>
         @page { size: 12px }
-        body { margin: 2px 0 0 2px; background: #fff; font-size: 0 }
+        body { margin: 2px 0 0 2px; font-size: 0 }
         img { display: block; width: 8px; image-rendering: pixelated }
       </style>
       <div><img src="%s"></div>''' % filename)
@@ -180,13 +184,11 @@ def test_resized_images(filename):
     ('0 0 4 4', 4, 4),
     ('0 0 4 4', 4, 4),
 ))
-def test_svg_sizing(viewbox, width, height):
-    assert_pixels(
-        f'svg_sizing_{viewbox}_{width}_{height}', 8, 8,
-        centered_image, '''
+def test_svg_sizing(assert_pixels, viewbox, width, height):
+    assert_pixels(centered_image, '''
       <style>
         @page { size: 8px }
-        body { margin: 2px 0 0 2px; background: #fff; font-size: 0 }
+        body { margin: 2px 0 0 2px; font-size: 0 }
         svg { display: block }
       </style>
       <svg %s %s %s>
@@ -210,12 +212,11 @@ def test_svg_sizing(viewbox, width, height):
     ('0 0 4 4', 8, 8, resized_image),
     ('0 0 4 4', 800, 800, resized_image),
 ))
-def test_svg_resizing(viewbox, width, height, image):
-    assert_pixels(
-        f'svg_resizing_{viewbox}_{width}_{height}', 12, 12, image, '''
+def test_svg_resizing(assert_pixels, viewbox, width, height, image):
+    assert_pixels(image, '''
       <style>
         @page { size: 12px }
-        body { margin: 2px 0 0 2px; background: #fff; font-size: 0 }
+        body { margin: 2px 0 0 2px; font-size: 0 }
         svg { display: block; width: 8px }
       </style>
       <svg %s %s %s>
@@ -228,23 +229,23 @@ def test_svg_resizing(viewbox, width, height, image):
 
 
 @assert_no_logs
-def test_images_block():
-    assert_pixels('block_image', 8, 8, centered_image, '''
+def test_images_block(assert_pixels):
+    assert_pixels(centered_image, '''
       <style>
         @page { size: 8px }
-        body { margin: 0; background: #fff; font-size: 0 }
+        body { margin: 0; font-size: 0 }
         img { display: block; margin: 2px auto 0 }
       </style>
       <div><img src="pattern.png"></div>''')
 
 
 @assert_no_logs
-def test_images_not_found():
+def test_images_not_found(assert_pixels):
     with capture_logs() as logs:
-        assert_pixels('image_not_found', 8, 8, no_image, '''
+        assert_pixels(no_image, '''
           <style>
             @page { size: 8px }
-            body { margin: 0; background: #fff; font-size: 0 }
+            body { margin: 0; font-size: 0 }
             img { display: block; margin: 2px auto 0 }
           </style>
           <div><img src="inexistent1.png" alt=""></div>''')
@@ -254,46 +255,42 @@ def test_images_not_found():
 
 
 @assert_no_logs
-def test_images_no_src():
-    assert_pixels('image_no_src', 8, 8, no_image, '''
+def test_images_no_src(assert_pixels):
+    assert_pixels(no_image, '''
       <style>
         @page { size: 8px }
-        body { margin: 0; background: #fff; font-size: 0 }
+        body { margin: 0; font-size: 0 }
         img { display: block; margin: 2px auto 0 }
       </style>
       <div><img alt=""></div>''')
 
 
 @assert_no_logs
-def test_images_alt():
+def test_images_alt(assert_same_renderings):
     with capture_logs() as logs:
-        assert_same_rendering(200, 30, [
-            (name, '''
+        documents = (
+            '''
               <style>
                 @page { size: 200px 30px }
-                body { margin: 0; background: #fff; font-size: 0 }
+                body { margin: 0; font-size: 0 }
               </style>
-              <div>%s</div>''' % html)
-            for name, html in [
-                ('image_alt_text_reference', 'Hello, world!'),
-                ('image_alt_text_not_found',
-                    '<img src="inexistent2.png" alt="Hello, world!">'),
-                ('image_alt_text_no_src',
-                    '<img alt="Hello, world!">'),
-                ('image_svg_no_intrinsic_size',
-                    '''<img src="data:image/svg+xml,<svg></svg>"
-                            alt="Hello, world!">'''),
-            ]
-        ])
+              <div>%s</div>''' % html
+            for html in (
+                'Hello',
+                '<img src="inexistent2.png" alt="Hello">',
+                '<img alt="Hello">',
+                '<img src="data:image/svg+xml,<svg></svg>" alt="Hello">',
+            ))
+        assert_same_renderings(*documents)
     assert len(logs) == 1
     assert 'ERROR: Failed to load image' in logs[0]
     assert 'inexistent2.png' in logs[0]
 
 
 @assert_no_logs
-def test_images_repeat_transparent():
+def test_images_repeat_transparent(assert_pixels):
     # Test regression: https://github.com/Kozea/WeasyPrint/issues/1440
-    assert_pixels('image_repeat_transparent', 1, 3, '_\n_\n_', '''
+    assert_pixels('_\n_\n_', '''
       <style>
         @page { size: 1px }
         div { height: 100px; width: 100px; background: url(logo_small.png) }
@@ -302,43 +299,43 @@ def test_images_repeat_transparent():
 
 
 @assert_no_logs
-def test_images_no_width():
-    assert_pixels('image_0x1', 8, 8, no_image, '''
+def test_images_no_width(assert_pixels):
+    assert_pixels(no_image, '''
       <style>
         @page { size: 8px }
-        body { margin: 2px; background: #fff; font-size: 0 }
+        body { margin: 2px; font-size: 0 }
       </style>
       <div><img src="pattern.png" alt="not shown"
                 style="width: 0; height: 1px"></div>''')
 
 
 @assert_no_logs
-def test_images_no_height():
-    assert_pixels('image_1x0', 8, 8, no_image, '''
+def test_images_no_height(assert_pixels):
+    assert_pixels(no_image, '''
       <style>
         @page { size: 8px }
-        body { margin: 2px; background: #fff; font-size: 0 }
+        body { margin: 2px; font-size: 0 }
       </style>
       <div><img src="pattern.png" alt="not shown"
                 style="width: 1px; height: 0"></div>''')
 
 
 @assert_no_logs
-def test_images_no_width_height():
-    assert_pixels('image_0x0', 8, 8, no_image, '''
+def test_images_no_width_height(assert_pixels):
+    assert_pixels(no_image, '''
       <style>
         @page { size: 8px }
-        body { margin: 2px; background: #fff; font-size: 0 }
+        body { margin: 2px; font-size: 0 }
       </style>
       <div><img src="pattern.png" alt="not shown"
                 style="width: 0; height: 0"></div>''')
 
 
 @assert_no_logs
-def test_images_page_break():
-    assert_pixels('image_page_break', 8, 3 * 8, page_break, '''
+def test_images_page_break(assert_pixels):
+    assert_pixels(page_break, '''
       <style>
-        @page { size: 8px; margin: 2px; background: #fff }
+        @page { size: 8px; margin: 2px }
         body { font-size: 0 }
       </style>
       <div><img src="pattern.png"></div>
@@ -346,11 +343,11 @@ def test_images_page_break():
 
 
 @assert_no_logs
-def test_image_repeat_inline():
+def test_image_repeat_inline(assert_pixels):
     # Test regression: https://github.com/Kozea/WeasyPrint/issues/808
-    assert_pixels('image_page_repeat_inline', 8, 2 * 8, table, '''
+    assert_pixels(table, '''
       <style>
-        @page { size: 8px; margin: 0; background: #fff }
+        @page { size: 8px; margin: 0 }
         table { border-collapse: collapse; margin: 2px }
         th, td { border: none; padding: 0 }
         th { height: 4px; line-height: 4px }
@@ -369,11 +366,11 @@ def test_image_repeat_inline():
 
 
 @assert_no_logs
-def test_image_repeat_block():
+def test_image_repeat_block(assert_pixels):
     # Test regression: https://github.com/Kozea/WeasyPrint/issues/808
-    assert_pixels('image_page_repeat_block', 8, 2 * 8, table, '''
+    assert_pixels(table, '''
       <style>
-        @page { size: 8px; margin: 0; background: #fff }
+        @page { size: 8px; margin: 0 }
         table { border-collapse: collapse; margin: 2px }
         th, td { border: none; padding: 0 }
         th { height: 4px }
@@ -392,11 +389,11 @@ def test_image_repeat_block():
 
 
 @assert_no_logs
-def test_images_padding():
+def test_images_padding(assert_pixels):
     # Regression test: padding used to be ignored on images
-    assert_pixels('image_with_padding', 8, 8, centered_image, '''
+    assert_pixels(centered_image, '''
       <style>
-        @page { size: 8px; background: #fff }
+        @page { size: 8px }
         body { font-size: 0 }
       </style>
       <div style="line-height: 1px">
@@ -405,12 +402,12 @@ def test_images_padding():
 
 
 @assert_no_logs
-def test_images_in_inline_block():
+def test_images_in_inline_block(assert_pixels):
     # Regression test: this used to cause an exception
-    assert_pixels('image_in_inline_block', 8, 8, centered_image, '''
+    assert_pixels(centered_image, '''
       <style>
         @page { size: 8px }
-        body { margin: 2px 0 0 2px; background: #fff; font-size: 0 }
+        body { margin: 2px 0 0 2px; font-size: 0 }
       </style>
       <div style="display: inline-block">
         <p><img src=pattern.png></p>
@@ -418,11 +415,11 @@ def test_images_in_inline_block():
 
 
 @assert_no_logs
-def test_images_shared_pattern():
+def test_images_shared_pattern(assert_pixels):
     # The same image is used in a repeating background,
     # then in a non-repating <img>.
     # If Pattern objects are shared carelessly, the image will be repeated.
-    assert_pixels('image_shared_pattern', 12, 12, '''
+    assert_pixels('''
         ____________
         ____________
         __aaaaaaaa__
@@ -438,7 +435,7 @@ def test_images_shared_pattern():
     ''', '''
       <style>
         @page { size: 12px }
-        body { margin: 2px; background: #fff; font-size: 0 }
+        body { margin: 2px; font-size: 0 }
       </style>
       <div style="background: url(blue.jpg);
                   height: 2px; margin-bottom: 1px"></div>
@@ -447,84 +444,137 @@ def test_images_shared_pattern():
 
 
 @assert_no_logs
-def test_image_resolution():
-    assert_same_rendering(20, 20, [
-        ('image_resolution_ref', '''
-            <style>@page { size: 20px; margin: 2px; background: #fff }</style>
+def test_image_resolution(assert_same_renderings):
+    assert_same_renderings(
+        '''
+            <style>@page { size: 20px; margin: 2px }</style>
             <div style="font-size: 0">
                 <img src="pattern.png" style="width: 8px"></div>
-        '''),
-        ('image_resolution_img', '''
-            <style>@page { size: 20px; margin: 2px; background: #fff }</style>
+        ''',
+        '''
+            <style>@page { size: 20px; margin: 2px }</style>
             <div style="image-resolution: .5dppx; font-size: 0">
                 <img src="pattern.png"></div>
-        '''),
-        ('image_resolution_content', '''
-            <style>@page { size: 20px; margin: 2px; background: #fff }
+        ''',
+        '''
+            <style>@page { size: 20px; margin: 2px }
                    div::before { content: url(pattern.png) }
             </style>
             <div style="image-resolution: .5dppx; font-size: 0"></div>
-        '''),
-        ('image_resolution_background', '''
-            <style>@page { size: 20px; margin: 2px; background: #fff }
+        ''',
+        '''
+            <style>@page { size: 20px; margin: 2px }
             </style>
             <div style="height: 16px; image-resolution: .5dppx;
                         background: url(pattern.png) no-repeat"></div>
-        '''),
-    ])
+        ''',
+    )
 
 
 @assert_no_logs
-def test_image_cover():
-    assert_pixels('image_cover', 8, 8, cover_image, '''
+def test_image_cover(assert_pixels):
+    assert_pixels(cover_image, '''
       <style>
         @page { size: 8px }
-        body { margin: 2px 0 0 2px; background: #fff; font-size: 0 }
+        body { margin: 2px 0 0 2px; font-size: 0 }
         img { object-fit: cover; height: 4px; width: 2px }
       </style>
       <img src="pattern.png">''')
 
 
 @assert_no_logs
-def test_image_contain():
-    assert_pixels('image_contain', 8, 8, centered_image, '''
+def test_image_contain(assert_pixels):
+    assert_pixels(centered_image, '''
       <style>
         @page { size: 8px }
-        body { margin: 1px 0 0 2px; background: #fff; font-size: 0 }
+        body { margin: 1px 0 0 2px; font-size: 0 }
         img { object-fit: contain; height: 6px; width: 4px }
       </style>
       <img src="pattern.png">''')
 
 
 @assert_no_logs
-def test_image_none():
-    assert_pixels('image_none', 8, 8, centered_image, '''
+def test_image_none(assert_pixels):
+    assert_pixels(centered_image, '''
       <style>
         @page { size: 8px }
-        body { margin: 1px 0 0 1px; background: #fff; font-size: 0 }
+        body { margin: 1px 0 0 1px; font-size: 0 }
         img { object-fit: none; height: 6px; width: 6px }
       </style>
       <img src="pattern.png">''')
 
 
 @assert_no_logs
-def test_image_scale_down():
-    assert_pixels('image_scale_down', 8, 8, centered_image, '''
+def test_image_scale_down(assert_pixels):
+    assert_pixels(centered_image, '''
       <style>
         @page { size: 8px }
-        body { margin: 1px 0 0 1px; background: #fff; font-size: 0 }
+        body { margin: 1px 0 0 1px; font-size: 0 }
         img { object-fit: scale-down; height: 6px; width: 6px }
       </style>
       <img src="pattern.png">''')
 
 
 @assert_no_logs
-def test_image_position():
-    assert_pixels('image_position', 8, 8, centered_image, '''
+def test_image_position(assert_pixels):
+    assert_pixels(centered_image, '''
       <style>
         @page { size: 8px }
-        body { margin: 1px 0 0 1px; background: #fff; font-size: 0 }
+        body { margin: 1px 0 0 1px; font-size: 0 }
         img { object-fit: none; height: 6px; width: 6px;
               object-position: bottom 50% right 50% }
       </style>
       <img src="pattern.png">''')
+
+
+@assert_no_logs
+def test_images_border(assert_pixels):
+    assert_pixels(border_image, '''
+      <style>
+        @page { size: 8px }
+        body { margin: 0; font-size: 0 }
+        img { margin: 1px; border: 1px solid lime }
+      </style>
+      <div><img src="pattern.png"></div>''')
+
+
+@assert_no_logs
+def test_images_border_absolute(assert_pixels):
+    assert_pixels(border_image, '''
+      <style>
+        @page { size: 8px }
+        body { margin: 0; font-size: 0 }
+        img { margin: 1px; border: 1px solid lime; position: absolute }
+      </style>
+      <div><img src="pattern.png"></div>''')
+
+
+@assert_no_logs
+def test_image_exif(assert_same_renderings):
+    assert_same_renderings(
+        '''
+            <style>@page { size: 10px }</style>
+            <img style="display: block" src="not-optimized.jpg">
+        ''',
+        '''
+            <style>@page { size: 10px }</style>
+            <img style="display: block" src="not-optimized-exif.jpg">
+        ''',
+        tolerance=25,
+    )
+
+
+@assert_no_logs
+def test_image_exif_image_orientation(assert_same_renderings):
+    assert_same_renderings(
+        '''
+            <style>@page { size: 10px }</style>
+            <img style="display: block; image-orientation: 180deg"
+                 src="not-optimized-exif.jpg">
+        ''',
+        '''
+            <style>@page { size: 10px }</style>
+            <img style="display: block" src="not-optimized-exif.jpg">
+        ''',
+        tolerance=25,
+    )
